/* type Mongo struct {
	Hosts         MongoHosts
	ClientOptions ClientOptions
	Oplog         ClientOptions
} */

// type DatabaseClientConfig

type AppBinding struct {
	ClientConfig
	Sources    settings.Sources
	Parameters *parameters.Parameters
}

type ServiceClientConfig struct {
	//	Secret / Auth
	// Specifies which scheme to use, for example: http, https
	// If specified, then it will applied as prefix in this format: scheme://
	// If not specified, then nothing will be prefixed
	Scheme string `json:"scheme" protobuf:"bytes,1,opt,name=scheme"`

	// `name` is the name of the service.
	// Required
	Name string `json:"name" protobuf:"bytes,2,opt,name=name"`

	// The port that will be exposed by this app.
	Port int32 `json:"port" protobuf:"varint,3,opt,name=port"`

	// `path` is an optional URL path which will be sent in any request to
	// this service.
	// +optional
	Path string `json:"path,omitempty" protobuf:"bytes,4,opt,name=path"`

	// `query` is optional encoded query string, without '?' which will be
	// sent in any request to this service.
	// +optional
	Query string `json:"query,omitempty" protobuf:"bytes,5,opt,name=query"`
}

type ClientConfig struct {
	url
	ServiceClientConfig
}

type MongoAppClient struct {
	Hosts         MongoHosts
	ClientOptions ClientOptions
	Oplog         ClientOptions
}

type MongoHosts struct {
	Hosts      *parameters.Parameter `json:"hosts,omitempty" env:"MONGO_HOSTS"`
	ServiceRef string                `json:"endpointRef,omitempty"`
	// Port       int32                 `json:"port,omitempty"`
}

type ClientOptions struct {
	Auth              *Credential
	Direct            *parameters.Parameter // *bool
	HeartbeatInterval *parameters.Parameter // *time.Duration
	//	Hosts             MongoHosts
	LocalThreshold *parameters.Parameter // *time.Duration

	Queries  *parameters.Parameter `json:"queries,omitempty" env:"MONGO_CLIENT_QUERIES "`
	Database *parameters.Parameter `json:"database,omitempty" env:"MONGO_DATABASE"`
}

type Credential struct {
	AuthMechanism           *parameters.Parameter // string
	AuthMechanismProperties *parameters.Parameter // map[string]string
	AuthSource              *parameters.Parameter // string
	Username                *parameters.Parameter // string
	Password                *parameters.Parameter // string
	//	PasswordSet             bool
}

// DefaultDNSSuffix is a default dns suffix for the cluster service
const DefaultDNSSuffix = "svc.cluster.local"

func InitEndpoint(c client.Client, ctx context.Context) {
	obj := &corev1.Endpoints{
		ObjectMeta: metav1.ObjectMeta{
			Name:      "",
			Namespace: "",
		},
	}

	objectKey, _ := client.ObjectKeyFromObject(obj)

	if err := c.Get(ctx, objectKey, obj); err != nil {
		//	log.Error(err, "unable to fetch endpoint")
		//	return ctrl.Result{}, ignoreNotFound(err)
	}

	hostnames := []string{}
	port := obj.Subsets[0].Ports[0].Port

	for _, addr := range obj.Subsets[0].Addresses {

		hostname := strings.Join([]string{addr.Hostname, "serviceName", obj.Namespace, DefaultDNSSuffix}, ".") +
			":" + strconv.Itoa(int(port))

		hostnames = append(hostnames, hostname)
	}
}

// hostname.subdomain.namespace.svc.cluster.local

func getAddr() string {
	//	return strings.Join([]string{pod, m.Name + "-" + replset, m.Namespace, m.Spec.ClusterServiceDNSSuffix}, ".") +
	//		":" + strconv.Itoa(int(m.Spec.Mongod.Net.Port))
	return ""
}
