/*

Licensed under the GNU AFFERO GENERAL PUBLIC LICENSE Version 3 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/agpl-3.0.html

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package job

import (
	batchv1 "k8s.io/api/batch/v1"

	"k8s.libre.sh/controller-utils/objects/pod"
)

type Mutate interface {
	GetParallelism() *int32
	GetCompletions() *int32
	GetActiveDeadlineSeconds() *int64
	GetBackoffLimit() *int32
	GetTTLSecondsAfterFinished() *int32
	pod.Mutate
}

func MutateJob(i Mutate, obj *batchv1.JobSpec) error {

	obj.Parallelism = i.GetParallelism()
	obj.Completions = i.GetCompletions()
	obj.ActiveDeadlineSeconds = i.GetActiveDeadlineSeconds()
	obj.BackoffLimit = i.GetBackoffLimit()
	obj.TTLSecondsAfterFinished = i.GetTTLSecondsAfterFinished()

	err := pod.MutatePod(i, &obj.Template)
	if err != nil {
		return err
	}

	return nil
}
